/****************************************************************************
*
*  FILE:          SSHOOK.C
*
*  DESCRIPTION:   Dynamic Link Library for Screen Saver.
*                 These functions form a DLL for a Windows Keyboard Hook
*                 function.
*
*                 This software is hereby placed in the public domain.
*                 Use and abuse it at your own risk!
*
*
*  AUTHOR:        Tom Wheeler
*                 31294 Morlock
*                 Livonia, Mich.  48152
*                 [72037,1742]
*
*****************************************************************************
*
*     DATE                                DESCRIPTION
*  ----------     -----------------------------------------------------------
*   10/11/90      Initial Development
*   11/02/90      Minor improvements to structure
*   03/12/90      Got Rid of 'MakeProcInstance' in favor of 'GetProcAddress'
*                 in function 'InitHook'.  This seems to be recommended by
*                 Microsoft as the preferred way for getting the address of
*                 a function exported in a DLL.  Replaced the WH_KEYBOARD
*                 hook with WH_GETMESSAGE hook.  This module now filters
*                 keystrokes and mouse movement.
*
****************************************************************************/

#include <windows.h>
#include "ss.h"

/******************************** VARIABLES ********************************/

HANDLE hInst = NULL;
HANDLE hUserHandle = NULL;
FARPROC lpfnMsgHook = NULL;
FARPROC lpfnOldMsgHook = NULL;

/******************************** FUNCTIONS ********************************/

/****************************************************************************
*
*  int FAR PASCAL LibMain(HANDLE hInstance,WORD wDataSeg,WORD cbHeapSize,
*                         LPSTR lpszCmdLine)
*
*  DESCRIPTION:   DLL Initialization function.  Called from LIBENTRY.ASM
*                 which is the startup code for the DLL.
*
*  INPUT:         hInstance      - This modules instance Handle
*                 wDataSeg       - This modules Data Segment
*                 cbHeapSize     - Size of heap (if any)
*                 lpszCmdLine    - Command line parameters
*
*  OUTPUT:        Returns 1 (TRUE) to indicate success
*
****************************************************************************/
int FAR PASCAL LibMain(HANDLE hInstance,WORD wDataSeg,WORD cbHeapSize,
                       LPSTR lpszCmdLine)
{
   hInst = hInstance;               /* store instance Handle               */
   if(cbHeapSize != 0)              /* if DLL data seg is MOVEABLE         */
      UnlockData(0);
   return(1);                       /* initialization successful           */
}

/****************************************************************************
*
*  int FAR PASCAL WEP(int nParameter)
*
*  DESCRIPTION:   DLL Shutdown function.  Called when DLL is terminating.
*
*  INPUT:         nParameter     -  Termination Code
*
*  OUTPUT:        Returns 1 (TRUE) to indicate success
*
****************************************************************************/
int FAR PASCAL WEP(int nParameter)
{
   return(1);
}

/****************************************************************************
*
*  DWORD FAR PASCAL GetMessageHook(int iCode,WORD wParam,LONG lParam)
*
*  DESCRIPTION:   Keyboard Hook function
*
*  INPUT:         iCode       - Windows Hook Code
*                 wParam      - Virtual Key Code
*                 lParam      - Status Bits
*
*  OUTPUT:        Calls DefHookProc after filtering out keyboard and mouse
*                 messages.
*
****************************************************************************/
DWORD FAR PASCAL GetMessageHook(int iCode,WORD wParam,LONG lParam)
{
   MSG FAR *lpMsg;

   /* check if filter should process message */
   if(iCode >= 0) {
	   lpMsg = (MSG FAR *)lParam;
      switch(lpMsg->message) {
         case WM_MOUSEMOVE:
         case WM_KEYUP:
         case WM_SYSKEYUP:
   	   case WM_LBUTTONUP:
         case WM_RBUTTONUP:
            PostMessage(hUserHandle,WM_HOOKKEY,0,0L);
            break;
         default:
            break;
      }
   }
   return DefHookProc(iCode,wParam,lParam,(FARPROC FAR *)&lpfnOldMsgHook);
}

/****************************************************************************
*
*  BOOL FAR PASCAL InitHook(HANDLE hWnd)
*
*  DESCRIPTION:   Externally callable function intended for initialization.
*                 Allows this module to get a Handle to the user window and
*                 installs the Keyboard filter hook.
*
*  INPUT:         hWnd        - Users window handle
*
*  OUTPUT:        Returns TRUE if successful, FALSE otherwise
*
****************************************************************************/
BOOL FAR PASCAL InitHook(HANDLE hWnd)
{
   if(!hUserHandle) {
      hUserHandle = hWnd;
      lpfnMsgHook = GetProcAddress(hInst,(LPSTR)"GetMessageHook");
      lpfnOldMsgHook = SetWindowsHook(WH_GETMESSAGE,lpfnMsgHook);
      return TRUE;
   }
   else {
      MessageBox(hWnd,"SSHOOK.DLL already active!","SSHOOK",MB_ICONASTERISK |
                 MB_OK);
      return FALSE;
   }
}

/****************************************************************************
*
*  BOOL FAR PASCAL KillMsgHook(void)
*
*  DESCRIPTION:   Externally callable function intended for clean-up.  Done
*                 here because of the documented uncertanty of whether the
*                 WEP will or won't be called.
*
*  INPUT:         hWnd        - NA
*
*  OUTPUT:        Returns TRUE
*
****************************************************************************/
BOOL FAR PASCAL KillHook(void)
{
   UnhookWindowsHook(WH_GETMESSAGE,lpfnMsgHook);
   return TRUE;
}
